/*
Linefollowing code by Gabriel Fornaeus
######
More information at http://hax0r.se
*/

// For the Pololu Qik motor controller
#include <CompactQik2s9v1.h>
#include <NewSoftSerial.h>
// Setup pins for the motor controller
#define rxPin 3
#define txPin 4
#define rstPin 5

NewSoftSerial mySerial = NewSoftSerial(rxPin,txPin);
CompactQik2s9v1 motor = CompactQik2s9v1(&mySerial,rstPin);

// Pololu QTR sensors
#include <PololuQTRSensors.h>
#define NUM_SENSORS   3     // Number of sensors used(actually five, but we'll skip the outer two)
#define TIMEOUT       2500  // Waits for 2500 us for sensor outputs to go low

// Sensors 0 through 5 are connected to digital pins 4 through 6, respectively
PololuQTRSensorsRC qtrrc((unsigned char[]) {  8, 9, 10, },
  NUM_SENSORS, TIMEOUT);
unsigned int sensorValues[NUM_SENSORS];

// Variables for PD control

// Proportional constant
float KP = 0.1;
// Derivative constant
float KD = 5.05;

// Base motor speeds
int M0 = 101;
int M1 = 98;

int lastError = 0;

void setup()
{
	// Output pins
	pinMode(3, OUTPUT);
	pinMode(4, OUTPUT);
	pinMode(5, OUTPUT);
	// Input pins
	pinMode(8, INPUT);
	pinMode(9, INPUT);
	pinMode(10, INPUT);

	// Setup motors
	mySerial.begin(9600);
	motor.begin();
	delay(200);
	motor.stopBothMotors();

  digitalWrite(13, HIGH);

  delay(300);
  int i;
  for (i = 0; i < 80; i++)
	{
	    if (i < 20 || i >= 60)
		{
	    	motor.motor0Forward(70);
	    	motor.motor1Reverse(75);
		}
		else
		{
	    	motor.motor0Reverse(70);
	    	motor.motor1Forward(75);
		}
    qtrrc.calibrate();  // Reads all sensors 10 times at 2500 us per read (i.e. ~25 ms per call)
	}
	// Stop when the calibration is done
	motor.stopBothMotors();
	// End of calibration period
	digitalWrite(13, LOW);

	motor.stopBothMotors();
}


void loop()
{
	// Get the position of the line
	int position = qtrrc.readLine(sensorValues,QTR_EMITTERS_ON);
	
	/* Compute the "error" from the line position.
	   Error will be 0 when we're dead center over the line, -1000 when we're to the left and 1000
	   if it's to the right */
	int error = position - 1000;

	// Set motor speed based on proportional and derivative PD terms
	int motorSpeed = KP * error + KD * (error - lastError);
	lastError = error;

	int m0Speed = constrain((M0 + motorSpeed),0,127);
	int m1Speed = constrain((M1 - motorSpeed),0,127);

	motor.motor0Forward(m0Speed);
	motor.motor1Forward(m1Speed);
}
